/*

This is a GENERIC owner drawing system for DC_XbpBrowse
Each browse cell can draw text, icons or bitmaps anywhere
within the cell by declaring each within a multi-dimensional
array

  [1] = <ocData>
  [2] = <nCol>
  [3] = <nRow>
  [4] = <nWidth>
  [5] = <nHeight>
  [6] = <font>
  [7] = <selectedFont>
  [8] = <fgColor>
  [9] = <bgColor>
 [10] = <nAlign>

*/

#INCLUDE "dcdialog.CH"

#define FONTBOLD      "Bold"    // change to "Fett" for German versions
#define FONTITALIC    "Italic"  // change to "Kursiv" for German versions
#define CRLF          Chr(13)+Chr(10)

CLASS XbpOwnerDrawBrowseGeneric FROM DC_XbpBrowse

PROTECTED:
   VAR oIcon, oBitmap, aLineAttrs, aBoxAttrs, oStdFont, oBoldFont, oBoldItalicFont, ;
       oFont, oSelectedFont

EXPORTED:

VAR lOwnerDraw, nLineHeight, nCellHeight
VAR stdFontCompoundName, boldFontCompoundName, boldItalicFontCompoundName

METHOD CustomDrawCell

INLINE METHOD destroy
   ::DC_XbpBrowse:destroy()
RETURN self

INLINE METHOD init( oParent, oOwner, aPos, aSize, aPP, lVisible, oGetList )

::drawMode:= XBP_DRAW_OWNER
::oIcon := XbpIcon():New():Create()
::oBitmap := XbpBitmap():New():Create()
::nLineHeight := 17.5
::nCellHeight := 20
::lOwnerDraw := .F.
::aLineAttrs := ARRAY( GRA_AL_COUNT )
::aBoxAttrs  := ARRAY( GRA_AA_COUNT )
::aLineAttrs [ GRA_AL_COLOR ] := GRA_CLR_BLACK
::aLineAttrs [ GRA_AL_TYPE ]  := GRA_LINETYPE_SOLID
::stdFontCompoundName := '10.Tahoma'
::boldFontCompoundName := '10.Tahoma '+FONTBOLD
::boldItalicFontCompoundName := '10.Tahoma '+FONTBOLD+' '+FONTITALIC
::DC_XbpBrowse:init( oParent, oOwner, aPos, aSize, aPP, lVisible, oGetList )

RETURN self

INLINE METHOD create( oParent, oOwner, aPos, aSize, aPP, lVisible )

LOCAL nFound

::oStdFont := XbpFont():New():Create(::stdFontCompoundName)
::oBoldFont := XbpFont():New():Create(::boldFontCompoundName)
::oBoldItalicFont := XbpFont():New():Create(::boldItalicFontCompoundName)
::oSelectedFont := ::oBoldItalicFont
::oFont := XbpFont():New():Create()
::DC_XbpBrowse:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

IF Empty(aPP)
  aPP := ::setPresParam()
ENDIF

nFound := AScan(aPP,{|a|Valtype(a[1])=='N'.AND. ;
                  a[1]==XBP_PP_COL_DA_CELLHEIGHT } )
IF nFound > 0
  ::nCellHeight := aPP[nFound,2]
ENDIF

RETURN self

ENDCLASS

* ----------

METHOD XbpOwnerDrawBrowseGeneric:customDrawCell( oPS, aInfo )

LOCAL aData, aPP, nFound, nAlign, nLine, nFgColor := GRA_CLR_BLACK, ;
      nBgColor := GRA_CLR_WHITE, oBitmap, aSource, aTarget, ;
      aIcons, nIcons, nHeight, nWidth, nLineHeight, cCaption, ;
      nStartRow, nStartCol, nEndRow, nEndCol, i, aLine, ocFont, ;
      oOldFont, aSize, lDisabled := .f., lIcon := .f., lBitmap := .f., ;
      lSelected, cFont, cSelectedFont, nCellHeight

IF !::lOwnerDraw // ownerdrawing is switched off, let the system draw
  RETURN .T.
ENDIF

* get column data
// aData := aInfo[XBP_DRAWINFO_AREA]:cargo
// IF Valtype(aData) # 'A'
  aData := aInfo[ XBP_DRAWINFO_AREA ]:getCell( aInfo[ XBP_DRAWINFO_ITEM ] )
// ENDIF

lSelected := BAnd(aInfo[XBP_DRAWINFO_STATE],XBP_DRAWSTATE_SELECTED) # 0 // cell is selected

* get cell alignment
aPP := aInfo[ XBP_DRAWINFO_COLUMN ]:presArray
nFound := AScan(aPP,{|a|Valtype(a[1])=='N'.AND. ;
                 (a[1]==XBPCOL_DA_CELLALIGNMENT .OR. ;
                  a[1]==XBP_PP_COL_DA_CELLALIGNMENT) } )

IF nFound > 0
  nAlign := aPP[nFound,2]
ELSE
  nAlign := XBPALIGN_VCENTER
ENDIF

* get cell height
nLineHeight := ;
   Int((aInfo[ XBP_DRAWINFO_RECT, 4 ]-aInfo[ XBP_DRAWINFO_RECT, 2 ])/(::nCellHeight/::nLineHeight))

* get cell colors
aInfo[ XBP_DRAWINFO_AREA ]:getCellColor( aInfo[ XBP_DRAWINFO_ITEM ],@nFgColor, @nBgColor )

IF nFgColor # nil
  oPS:setColor( nFgColor, nBgColor )
ENDIF

IF Valtype(aData) == 'C'
  * no line break, just paint the text
  GraCaptionStr( oPS, aInfo[ XBP_DRAWINFO_RECT ], ;
                     { aInfo[ XBP_DRAWINFO_RECT, 3 ], ;
                       aInfo[ XBP_DRAWINFO_RECT, 4 ] }, aData, nAlign )
ENDIF

IF Valtype(aData) # 'A'
  RETURN .f.
ENDIF

FOR i := 1 TO Len(aData)

  aLine := aData[i]
  IF Valtype(aLine) # 'A'
    LOOP
  ENDIF
  ASize(aLine,10)

  /*
  [1] = <ocData>
  [2] = <nCol>
  [3] = <nRow>
  [4] = <nWidth>
  [5] = <nHeight>
  [6] = <font>
  [7] = <selectedFont>
  [8] = <fgColor>
  [9] = <bgColor>
 [10] = <nAlign>
  */

  lIcon := .f.
  lBitmap := .f.
  ocFont := nil
  cCaption := aLine[1]
  nStartCol := aLine[2]
  IF Valtype(nStartCol) # 'N'
    nStartCol := aInfo[ XBP_DRAWINFO_RECT,1 ]
  ENDIF
  nStartRow := aLine[3]
  IF Valtype(nStartRow) # 'N'
    nStartRow := aInfo[ XBP_DRAWINFO_RECT,4 ]-(nLineHeight*(i-1))
  ELSE
    nStartRow := aInfo[ XBP_DRAWINFO_RECT,4 ]-nStartRow
  ENDIF
  nWidth := aLine[4]
  IF Valtype(nWidth) # 'N'
    nEndCol := aInfo[ XBP_DRAWINFO_RECT, 3 ]
  ELSE
    nEndCol := nStartCol + nWidth
  ENDIF
  nHeight := aLine[5]
  IF Valtype(nHeight) # 'N'
    nEndRow := aInfo[ XBP_DRAWINFO_RECT, 4 ]-(nLineHeight*i)
  ELSE
    nEndRow := nStartRow + nHeight
  ENDIF
  cFont := aLine[6]
  cSelectedFont := aLine[7]
  IF Empty(cSelectedFont) .AND. !Empty(cFont)
    cSelectedFont := cFont
  ENDIF
  IF Empty(cFont)
    ocFont := ::oStdFont
  ELSEIF Valtype(cFont) == 'C' .AND. !lSelected
    ocFont := ::oFont
    ocFont:configure(cFont)
  ELSEIF lSelected .AND. Valtype(cSelectedFont) == 'C'
    ocFont := ::oFont
    ocFont:configure(cSelectedFont)
  ENDIF

  nFGColor := aLine[8]
  nBGColor := aLine[9]
  nAlign := aLine[10]

  IF Valtype(nFGColor) == 'A'
    nFGColor := GraMakeRGBColor(nFGColor)
  ENDIF

  IF Valtype(nBGColor) == 'A'
    nBGColor := GraMakeRGBColor(nBGColor)
  ENDIF

  aSize := { nEndCol-nStartCol, nEndRow-nStartRow }

  IF Valtype(nFGColor) == 'N' .OR. nBGColor == 'N'
    GraSetColor( oPS, nFGColor, nBGColor )
  ENDIF

  IF Valtype(cCaption) == 'N'
    oBitmap := DC_GetBitmap(cCaption)
    IF Valtype(oBitmap) == 'O' .AND. oBitmap:isDerivedFrom('XbpBitmap')
      ::oBitmap := oBitmap
      lBitmap := .t.
    ELSEIF Valtype(oBitmap) == 'O' .AND. oBitmap:isDerivedFrom('XbpIcon')
      ::oIcon := oBitmap
      lIcon := .t.
    ENDIF
  ELSEIF Right(Upper(cCaption),4) == '.ICO'
    ::oIcon:loadFile(cCaption,nLineHeight,nLineHeight)
    lIcon := .t.
  ELSEIF Right(Upper(cCaption),4) $ {'.BMP','.JPG', '.PNG', '.GIF'}
    ::oBitmap:loadFile(cCaption)
    lBitmap := .t.
  ENDIF

  IF Valtype(ocFont) == 'O' .AND. !lIcon .AND. !lBitmap
    oOldFont := GraSetFont( oPS, ocFont )
  ENDIF

  IF lIcon

    aTarget := { nStartCol, nStartRow, nEndCol, nEndRow }
    ::oIcon:Draw( oPS, aTarget, XBP_STATE_NORMAL )

  ELSEIF lBitmap

    aSource := {0,0,oBitmap:xSize,oBitmap:ySize}

    aTarget := { nStartCol, nStartRow, nEndCol, nEndRow }

    ::oBitmap:draw( oPS, aTarget, aSource, GRA_BLT_ROP_SRCCOPY,, XBP_STATE_NORMAL )

  ELSEIF Valtype( cCaption ) == 'C'

    GraCaptionStr( oPS, { nStartCol, nStartRow }, ;
                        { nEndCol, nEndRow }, cCaption, nAlign )

  ENDIF

  IF Valtype(oOldFont) = 'O'
    GraSetFont( oPS, oOldFont )
  ENDIF

  IF !lBitMap .AND. !lIcon
    GraSetColor( oPS, nFGColor )
  ENDIF

NEXT

RETURN .F.


